using System;
using System.Web.UI;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;
using System.IO;
using System.Collections.Generic;

namespace CSSamples.Samples.Labeling
{
    public partial class LabelPointsOfInterest : System.Web.UI.Page
    {
        private static Collection<PointShape> pointShapes1;
        private static Collection<PointShape> pointShapes2;
        private int count1 = 0;
        private int count2 = 0;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.MapUnit = GeographyUnit.DecimalDegree;

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(MapPath("~/SampleData/world/cntry02.shp"));
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(255, 243, 239, 228), GeoColor.FromArgb(255, 218, 193, 163), 1);
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
                inMemoryFeatureLayer.Open();
                inMemoryFeatureLayer.Columns.Add(new FeatureSourceColumn("Angle"));
                inMemoryFeatureLayer.Close();
                pointShapes1 = ReadTextFile(Server.MapPath(@"~\Data\GPSreadings1.txt"));
                pointShapes2 = ReadTextFile(Server.MapPath(@"~\Data\GPSreadings2.txt"));

                Feature newFeature1 = new Feature(pointShapes1[count1]);
                newFeature1.ColumnValues["Angle"] = System.Convert.ToString(pointShapes1[count1].Z);
                inMemoryFeatureLayer.InternalFeatures.Add("Vehicle1", newFeature1);

                Feature newFeature2 = new Feature(pointShapes2[count2]);
                newFeature2.ColumnValues["Angle"] = System.Convert.ToString(pointShapes2[count2].Z);
                inMemoryFeatureLayer.InternalFeatures.Add("Vehicle2", newFeature2);

                RotatedImageStyle rotatedImageStyle = new RotatedImageStyle(new GeoImage(Server.MapPath(@"~\Data\vehicle2.png")), "Angle");
                inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear();
                inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(rotatedImageStyle);
                inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                ShapeFileFeatureLayer majorCitiesShapeLayer = new ShapeFileFeatureLayer(Server.MapPath(@"~\SampleData\USA\cities_a.shp"));
                majorCitiesShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City3;
                majorCitiesShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City3("AREANAME");
                majorCitiesShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                majorCitiesShapeLayer.DrawingMarginPercentage = 50;

                Map1.StaticOverlay.Layers.Add(worldLayer);
                Map1.StaticOverlay.Layers.Add(majorCitiesShapeLayer);

                Map1.DynamicOverlay.Layers.Add(inMemoryFeatureLayer);
                inMemoryFeatureLayer.Open();
                Map1.CurrentExtent = inMemoryFeatureLayer.GetBoundingBox();
            }
        }

        private Collection<PointShape> ReadTextFile(string textFile)
        {
            Collection<PointShape> pointShapes = new Collection<PointShape>();
            StreamReader SR = File.OpenText(textFile);
            string S;
            S = SR.ReadLine();
            while (S != null)
            {
                double X = System.Convert.ToDouble(S.Substring(0, 8));
                double Y = System.Convert.ToDouble(S.Substring(9, 7));
                double Angle = System.Convert.ToDouble(S.Substring(17, 3));
                PointShape pointShape = new PointShape(X, Y, Angle);
                pointShapes.Add(pointShape);
                S = SR.ReadLine();

            }
            SR.Close();
            return pointShapes;
        }

    }

    public class RotatedImageStyle : Style
    {
        private GeoImage geoImage;
        private string angleColumnName;

        public RotatedImageStyle()
            : this(new GeoImage(), string.Empty)
        { }

        public RotatedImageStyle(GeoImage geoImage, string angleColumnName)
        {
            this.geoImage = geoImage;
            this.angleColumnName = angleColumnName;
        }

        public GeoImage GeoImage
        {
            get { return geoImage; }
            set { GeoImage = value; }
        }

        public string AngleColumnName
        {
            get { return angleColumnName; }
            set { AngleColumnName = value; }
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            foreach (Feature feature in features)
            {
                float angleData = Convert.ToSingle(feature.ColumnValues[angleColumnName]);
                PointShape pointShape = (PointShape)feature.GetShape();
                canvas.DrawWorldImageWithoutScaling(geoImage, pointShape.X, pointShape.Y, DrawingLevel.LevelFour, 0, 0, 360 - angleData); //angleData);
            }
        }

        protected override Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> columns = new Collection<string>();
            if (!columns.Contains(angleColumnName))
            {
                columns.Add(angleColumnName);
            }
            return columns;
        }
    }
}
